# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2022/1/24 17:14
# @Author : BruceLong
# @FileName: x_es.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import hashlib
import itertools
import json
import math

import pandas as pd
from elasticsearch import Elasticsearch

from .config import ES_HOST_PORT_LIST
from .x_single import SingletonType


class XES(metaclass=SingletonType):
    def __init__(self, host_port_list=ES_HOST_PORT_LIST):
        self.host_port_list = host_port_list.split(';') if ES_HOST_PORT_LIST else []
        self.index = None
        self.batch_size = 50000
        self.max_request_timeout = 60
        self.batch_id = ''
        self.aggs_result_sign = 0
        self.aggs_result = []
        self.total_count = 0
        self.total_count_sign = 0
        self.agg_name = ''
        pass

    def connect(self):
        es = Elasticsearch(self.host_port_list)
        return es

    # def query(self, index: str, query_json: str, to_excel: bool = False):
    def query(self, query_json: str, to_excel: bool = False):
        es = self.connect()
        self.batch_id = hashlib.md5((str(query_json)).encode('utf8')).hexdigest()
        query_body = json.loads(query_json)
        size = query_body.get('size')
        hits_result = itertools.chain()
        if size >= self.batch_size:
            times = math.ceil(size / self.batch_size)
            for t in range(1, times + 1):
                query_body['size'] = self.batch_size * t
                result_temp = self.__get_data(es, query_body=query_body)
                hits_result = itertools.chain(hits_result, result_temp)
        else:
            hits_result = self.__get_data(es, query_body=query_body)
        return_hits_result, temp_hits_result = itertools.tee(hits_result, 2)

        if to_excel:
            self.__init_excel()
            df = pd.DataFrame(list(temp_hits_result))
            df.fillna('', inplace=True)
            df.to_excel(self.writer, encoding='utf_8_sig', sheet_name='hits_result')
            if self.aggs_result_sign:
                df = pd.DataFrame(list(self.aggs_result))
                df.fillna('', inplace=True)
                df.to_excel(self.writer, encoding='utf_8_sig', sheet_name=self.agg_name + 'aggs_result')
            self.writer.save()

        return return_hits_result
        pass

    def __get_data(self, es, query_body):
        contents = es.search(index=self.index, body=query_body, request_timeout=self.max_request_timeout)
        hits_result = (i['_source'] for i in contents.get('hits').get('hits') if i)
        if not self.total_count_sign:
            self.total_count = contents.get('hits').get('total')
        aggs_data = contents.get('aggregations')
        if aggs_data and not self.aggs_result_sign:
            self.agg_name = '_'.join(list(aggs_data.keys()))
            self.aggs_result = (data for i in aggs_data.keys() for data in aggs_data[i]['buckets'])
            self.aggs_result_sign = 1
        return hits_result

    def __init_excel(self):
        self.file_name = f'es_result_{self.batch_id}.xlsx'
        self.writer = pd.ExcelWriter(self.file_name, engine='xlsxwriter')



# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2022/1/24 16:38
# @Author : BruceLong
# @FileName: x_kafka.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import json

from kafka import KafkaProducer
from kafka.errors import KafkaError
from .config import KAFKA_HOST, KAFKA_PORT, KAFKA_TOPIC
from .x_single import SingletonType


class XKafka(metaclass=SingletonType):

    def __init__(self, host: str = KAFKA_HOST, port: str = KAFKA_PORT, kafka_topic=KAFKA_TOPIC):
        self.host = host
        self.port = port
        self.kafka_topic = kafka_topic
        self.__producer = KafkaProducer(bootstrap_servers='{kafka_host}:{kafka_port}'.format(
            kafka_host=self.host,
            kafka_port=self.port,
        ))

    def insert(self, item, **kwargs):
        '''
        topic, value=None, key=None, partition=None, timestamp_ms=None
        :param item:
        :param kwargs:
        :return:
        '''
        parmas_message = json.dumps(dict(item))
        try:
            self.__producer.send(topic=self.kafka_topic, value=parmas_message.encode('utf-8'), **kwargs)
            self.__producer.flush()
            return item
        except KafkaError as e:
            raise e


x_kafka = XKafka()



# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2022/1/24 17:16
# @Author : BruceLong
# @FileName: x_single.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import threading


class SingletonType(type):
    _instance_lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        if not hasattr(cls, "_instance"):
            with SingletonType._instance_lock:
                if not hasattr(cls, "_instance"):
                    cls._instance = super(SingletonType, cls).__call__(*args, **kwargs)
        return cls._instance



# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2021/12/29 11:20
# @Author : BruceLong
# @FileName: setup.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
from setuptools import setup, find_packages
import os

# 如果readme文件中有中文，那么这里要指定encoding='utf-8'，否则会出现编码错误
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as readme:
    README = readme.read()

# 允许setup.py在任何路径下执行
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="xdbx",
    version="1.0.0",
    packages=find_packages(),
    long_description=README,  # 详细描述（一般会写在README.md中）
    long_description_content_type="text/markdown",  # README.md中描述的语法（一般为markdown）
    url="https://gitee.com/spider-x/xdbx",
    author="BruceLong",
    license="MIT Licence",
    author_email="18656170559@163.com",
    include_package_data=True,
    platforms="any",
    install_requires=['wheel', 'pymysql', 'pymssql', 'dbutils', 'pyxbox>=0.0.3', 'kafka', 'elasticsearch', 'pandas',
                      'xlsxwriter']
)
